import numpy as np

from gui import GridLayer
from gui.opengl.OpenGLMesh import OpenGLMesh


class OpenGLGridLayer(OpenGLMesh, GridLayer):

    def setTiles(self, grid: np.ndarray):
        assert grid.ndim == 3
        gridWidth = grid.shape[0]
        gridHeight = grid.shape[1]
        assert grid.shape[2] == 2

        vertices = np.empty([gridWidth, gridHeight, 4, 2], dtype=np.float32)
        uvMap = np.empty([gridWidth, gridHeight, 4, 2], dtype=np.float32)
        faces = np.empty([gridWidth, gridHeight, 4], dtype=np.uint)
        faceCount = 0
        for y in range(gridHeight):
            for x in range(gridWidth):
                spriteScreenX1 = -1 + x * self.screenTileWidth
                spriteScreenY1 = 1 - y * self.screenTileHeight
                spriteScreenX2 = spriteScreenX1 + self.screenTileWidth
                spriteScreenY2 = spriteScreenY1 - self.screenTileHeight
                vertices[x, y, 0] = [spriteScreenX1, spriteScreenY2]
                vertices[x, y, 1] = [spriteScreenX1, spriteScreenY1]
                vertices[x, y, 2] = [spriteScreenX2, spriteScreenY1]
                vertices[x, y, 3] = [spriteScreenX2, spriteScreenY2]

                spriteTextureX1 = grid[x, y, 0] * self.textureTileWidth
                spriteTextureY1 = grid[x, y, 1] * self.textureTileHeight
                spriteTextureX2 = spriteTextureX1 + self.textureTileWidth
                spriteTextureY2 = spriteTextureY1 + self.textureTileHeight
                uvMap[x, y, 0] = [spriteTextureX1, spriteTextureY2]
                uvMap[x, y, 1] = [spriteTextureX1, spriteTextureY1]
                uvMap[x, y, 2] = [spriteTextureX2, spriteTextureY1]
                uvMap[x, y, 3] = [spriteTextureX2, spriteTextureY2]

                faces[x, y, 0] = faceCount * 4
                faces[x, y, 1] = faceCount * 4 + 1
                faces[x, y, 2] = faceCount * 4 + 2
                faces[x, y, 3] = faceCount * 4 + 3
                faceCount += 1

        self.setData(vertices, faces, uvMap)

