import logging

from PIL import Image

from gui import GUIFacadeFactory, KeyEvent, Keyboard, TextLayer
from misc import LevelLoader

logging.basicConfig(level=logging.INFO, format='\r%(filename)s:%(lineno)d: %(message)s')

# Create window
guiFacade = GUIFacadeFactory().createInstance("OpenGL")
guiFacade.createWindow(
    "OpenGL 2D Facade - https://www.patternsgameprog.com/",
    1280, 768
)

# List of text layers and messages
textLayers = []
messages = []

# Normal text Layer
textLayer = guiFacade.createTextLayer()
textLayer.setFont("assets/font/terminal-grotesque.ttf", 64, (255, 255, 255), TextLayer.PLAIN)
textLayer.setMaxCharacterCount(100)
textLayers.append(textLayer)
messages.append("Text styles with OpenGL")

# Bold text Layer
textLayer = guiFacade.createTextLayer()
textLayer.setFont("assets/font/terminal-grotesque.ttf", 64, (255, 255, 255), TextLayer.BOLD)
textLayer.setMaxCharacterCount(100)
textLayers.append(textLayer)
messages.append("Bold")

# Italic text Layer
textLayer = guiFacade.createTextLayer()
textLayer.setFont("assets/font/terminal-grotesque.ttf", 64, (255, 255, 255), TextLayer.ITALIC)
textLayer.setMaxCharacterCount(100)
textLayers.append(textLayer)
messages.append("Italic")

# Underline text Layer
textLayer = guiFacade.createTextLayer()
textLayer.setFont("assets/font/terminal-grotesque.ttf", 64, (255, 255, 255), TextLayer.UNDERLINE)
textLayer.setMaxCharacterCount(100)
textLayers.append(textLayer)
messages.append("Underline")

# Bold and italic text Layer
textLayer = guiFacade.createTextLayer()
textLayer.setFont("assets/font/terminal-grotesque.ttf", 64, (255, 255, 255), TextLayer.BOLD | TextLayer.ITALIC)
textLayer.setMaxCharacterCount(100)
textLayers.append(textLayer)
messages.append("Bold italic")

# Shadow text Layer
textLayer = guiFacade.createTextLayer()
textLayer.setShadowColor((255, 0, 0))
textLayer.setFont("assets/font/terminal-grotesque.ttf", 64, (255, 255, 255), TextLayer.SHADOW)
textLayer.setMaxCharacterCount(100)
textLayers.append(textLayer)
messages.append("Shadow")

# Shadow underline text Layer
textLayer = guiFacade.createTextLayer()
textLayer.setShadowColor((255, 0, 0))
textLayer.setFont("assets/font/terminal-grotesque.ttf", 64, (255, 255, 255), TextLayer.SHADOW | TextLayer.UNDERLINE)
textLayer.setMaxCharacterCount(100)
textLayers.append(textLayer)
messages.append("Shadow underline")

# Outline text Layer
textLayer = guiFacade.createTextLayer()
textLayer.setOutlineColor((255, 0, 0))
textLayer.setFont("assets/font/terminal-grotesque.ttf", 64, (255, 255, 255), TextLayer.OUTLINE)
textLayer.setMaxCharacterCount(100)
textLayers.append(textLayer)
messages.append("Outline")

# URL text Layer
textLayer = guiFacade.createTextLayer()
textLayer.setOutlineColor((255, 0, 0))
textLayer.setFont("assets/font/terminal-grotesque.ttf", 64, (255, 255, 255), TextLayer.UNDERLINE)
textLayer.setMaxCharacterCount(100)
textLayers.append(textLayer)
messages.append("https://www.patternsgameprog.com")

messageCurrentIndex = 0

# Main loop
guiFacade.init()
frameCount = 0
while not guiFacade.closingRequested:
    # Update inputs state
    guiFacade.updateInputs()

    # Keyboard key single press
    keyboard = guiFacade.keyboard
    for keyEvent in keyboard.keyEvents:
        if keyEvent.type == KeyEvent.KEYDOWN:
            if keyEvent.key == Keyboard.K_ESCAPE:
                guiFacade.closingRequested = True
                break
    keyboard.clearKeyEvents()

    # Update text every 3 frames
    if (frameCount % 3) == 0:
        for i, textLayer in enumerate(textLayers):
            textLayer.setText(1 * 32, (i + 1) * 64, messages[i][0:messageCurrentIndex])

        messageCurrentIndex += 1
        if messageCurrentIndex >= 50:
            messageCurrentIndex = 0

    # Render scene
    guiFacade.render()
    frameCount += 1

if False:  # Screen capture (to check that the rendering is ok)
    capture = guiFacade.captureScreenContent(0, 0, guiFacade.screenWidth, guiFacade.screenHeight)
    Image.fromarray(capture).save("capture.png")

# Delete all objects and quit
guiFacade.quit()
